@@ -1,7 +1,7 @@
 #
 # This file is part of TBX-Min
 #
-# This software is copyright (c) 2013 by Alan Melby.
+# This software is copyright (c) 2016 by Alan Melby.
 #
 # This is free software; you can redistribute it and/or modify it under
 # the same terms as the Perl 5 programming language system itself.
@@ -9,25 +9,41 @@
 package TBX::Min;
 use strict;
 use warnings;
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 # ABSTRACT: Read, write and edit TBX-Min files
+use subs qw(date_created directionality entries);
+use Class::Tiny qw(
+    id
+    description
+    creator
+    license
+    source_lang
+    target_lang
+    date_created
+    directionality
+    entries
+);
 use XML::Twig;
 use autodie;
 use Path::Tiny;
 use Carp;
-use TBX::Min::Entry;
-use TBX::Min::LangGroup;
-use TBX::Min::TermGroup;
 use Import::Into;
 use DateTime::Format::ISO8601;
 use Try::Tiny;
+use TBX::Min::TermEntry;
+use TBX::Min::LangSet;
+use TBX::Min::TIG;
+use TBX::Min::NoteGrp;
+use TBX::Min::Note;
 
 # Use Import::Into to export subclasses into caller
 sub import {
     my $target = caller;
-    TBX::Min::Entry->import::into($target);
-    TBX::Min::LangGroup->import::into($target);
-    TBX::Min::TermGroup->import::into($target);
+    TBX::Min::TermEntry->import::into($target);
+    TBX::Min::LangSet->import::into($target);
+    TBX::Min::TIG->import::into($target);
+    TBX::Min::NoteGrp->import::into($target);
+    TBX::Min::Note->import::into($target);
     return;
 }
 
@@ -46,11 +62,13 @@ sub new_from_xml {
         # do_not_chain_handlers => 1, #can be important when things get complicated
         keep_spaces     => 0,
 
-        # these store new entries, langGroups and termGroups
+        # these store new entries, langSets and tigs
         start_tag_handlers => {
-            entry => \&_conceptStart,
-            langGroup => \&_langStart,
-            termGroup => \&_termGrpStart,
+            termEntry => \&_termEntryStart,
+            langSet => \&_langStart,	#langset
+            tig => \&_termGrpStart, #tig
+            noteGrp => \&_noteGrpStart, #note group
+            note => \&_noteStart, #note
         },
 
         TwigHandlers    => {
@@ -64,19 +82,25 @@ sub new_from_xml {
             directionality => \&_directionality,
             languages => \&_languages,
 
-            # becomes part of the current TBX::Min::Entry object
+            # becomes part of the current TBX::Min::TermEntry object
             subjectField => sub {
                 shift->{tbx_min_entries}->[-1]->subject_field($_->text)},
 
-            # these become attributes of the current TBX::Min::TermGroup object
+            # these become attributes of the current TBX::Min::TIG object
             term => sub {shift->{tbx_min_current_term_grp}->term($_->text)},
             partOfSpeech => sub {
                 shift->{tbx_min_current_term_grp}->part_of_speech($_->text)},
-            note => sub {shift->{tbx_min_current_term_grp}->note($_->text)},
             customer => sub {
                 shift->{tbx_min_current_term_grp}->customer($_->text)},
             termStatus => sub {
                 shift->{tbx_min_current_term_grp}->status($_->text)},
+
+            # these become attributes of the current TBX::Min::Note object
+			noteKey => sub {shift->{tbx_min_current_note}->noteKey($_->text)},
+			noteValue => sub {shift->{tbx_min_current_note}->noteValue($_->text)},
+
+            # delete the termEntry twig when finished to free up memory
+            termEntry => sub {$_[0]->purge},
         }
     );
 
@@ -99,17 +123,30 @@ sub _get_handle {
     return $fh;
 }
 
+
+my %valid = map +($_=>1), Class::Tiny->get_all_attributes_for(__PACKAGE__);
 sub new {
     my ($class, $args) = @_;
     my $self;
     if((ref $args) eq 'HASH'){
-        #don't store a plain string for datetime
-        if(my $dt_string = $args->{date_created}){
-            $args->{date_created} = _parse_datetime($dt_string);
+
+        # validate arguments
+        if(my @invalids = grep !$valid{$_}, sort keys %$args){
+            croak 'Invalid attributes for class: ' .
+                join ' ', @invalids
+        }
+        if($args->{entries} && ref $args->{entries} ne 'ARRAY'){
+            croak q{Attribute 'entries' should be an array reference};
         }
         if(exists $args->{directionality}){
             _validate_dir($args->{directionality});
         }
+
+        # validate datetime and store object, not string
+        if(my $dt_string = $args->{date_created}){
+            $args->{date_created} = _parse_datetime($dt_string);
+        }
+
         $self = $args;
     }else{
         $self = {};
@@ -118,22 +155,6 @@ sub new {
     return bless $self, $class;
 }
 
-sub id {
-    my ($self, $id) = @_;
-    if($id) {
-        return $self->{id} = $id;
-    }
-    return $self->{id};
-}
-
-sub description {
-    my ($self, $description) = @_;
-    if($description) {
-        return $self->{description} = $description;
-    }
-    return $self->{description};
-}
-
 sub date_created {
     my ($self, $date_created) = @_;
     if($date_created) {
@@ -157,22 +178,6 @@ sub _parse_datetime {
     return $dt;
 }
 
-sub creator {
-    my ($self, $creator) = @_;
-    if($creator) {
-        return $self->{creator} = $creator;
-    }
-    return $self->{creator};
-}
-
-sub license {
-    my ($self, $license) = @_;
-    if($license) {
-        return $self->{license} = $license;
-    }
-    return $self->{license};
-}
-
 sub directionality {
     my ($self, $directionality) = @_;
     if(defined $directionality) {
@@ -190,23 +195,6 @@ sub _validate_dir {
     return;
 }
 
-
-sub source_lang {
-    my ($self, $source_lang) = @_;
-    if($source_lang) {
-        return $self->{source_lang} = $source_lang;
-    }
-    return $self->{source_lang};
-}
-
-sub target_lang {
-    my ($self, $target_lang) = @_;
-    if($target_lang) {
-        return $self->{target_lang} = $target_lang;
-    }
-    return $self->{target_lang};
-}
-
 sub entries { ## no critic(RequireArgUnpacking)
     my ($self) = @_;
     if (@_ > 1){
@@ -216,11 +204,11 @@ sub entries { ## no critic(RequireArgUnpacking)
 }
 
 sub add_entry {
-    my ($self, $entry) = @_;
-    if( !$entry || !$entry->isa('TBX::Min::Entry') ){
-        croak 'argument to add_entry should be a TBx::Min::Entry';
+    my ($self, $termEntry) = @_;
+    if( !$termEntry || !$termEntry->isa('TBX::Min::TermEntry') ){
+        croak 'argument to add_entry should be a TBx::Min::TermEntry';
     }
-    push @{$self->{entries}}, $entry;
+    push @{$self->{entries}}, $termEntry;
     return;
 }
 
@@ -251,21 +239,25 @@ sub as_xml {
     }
 
     my $body = XML::Twig::Elt->new('body')->paste(last_child => $root);
-    for my $entry (@{$self->entries}){
+    for my $termEntry (@{$self->entries}){
         my $entry_el = XML::Twig::Elt->new(
-            entry => {$entry->id ? (id => $entry->id) : ()})->
+            termEntry => {$termEntry->id ? (id => $termEntry->id) : ()})->
             paste(last_child => $body);
-        if(my $sf = $entry->subject_field){
+        my $entry_el_comment = XML::Twig::Elt->new( '#COMMENT', 'terminological entry')->
+            paste(last_child => $entry_el);
+        if(my $sf = $termEntry->subject_field){
             XML::Twig::Elt->new(subjectField => $sf)->paste(
                 last_child => $entry_el);
         }
-        for my $langGrp (@{$entry->lang_groups}){
-            my $lang_el = XML::Twig::Elt->new(langGroup =>
+        for my $langGrp (@{$termEntry->lang_groups}){
+            my $lang_el = XML::Twig::Elt->new(langSet =>
                 {$langGrp->code ? ('xml:lang' => $langGrp->code) : ()}
             )->paste(last_child => $entry_el);
             for my $termGrp (@{$langGrp->term_groups}){
-                my $term_el = XML::Twig::Elt->new('termGroup')->paste(
+                my $term_el = XML::Twig::Elt->new('tig')->paste(
                     last_child => $lang_el);
+                my $term_el_comment = XML::Twig::Elt->new( '#COMMENT', 'terminological information group')->
+					paste(last_child => $term_el);
                 if (my $term = $termGrp->term){
                     XML::Twig::Elt->new(term => $term)->paste(
                         last_child => $term_el);
@@ -276,9 +268,25 @@ sub as_xml {
                         last_child => $term_el);
                 }
 
-                if (my $note = $termGrp->note){
-                    XML::Twig::Elt->new(note => $note)->paste(
-                        last_child => $term_el);
+				for my $noteGrp (@{$termGrp->note_groups}){
+					my $note_grp_el = XML::Twig::Elt->new('noteGrp')->paste(
+                    last_child => $term_el);
+
+					for my $note (@{$noteGrp->notes}){
+						my $note_el = XML::Twig::Elt->new('note')->paste(
+                    last_child => $note_grp_el);
+
+						if (my $noteKey = $note->noteKey){
+							XML::Twig::Elt->new(noteKey => $noteKey)->paste(
+								last_child => $note_el);
+						}
+
+						if (my $noteValue = $note->noteValue){
+							XML::Twig::Elt->new(noteValue => $noteValue)->paste(
+								last_child => $note_el);
+						}
+
+					}
                 }
 
                 if (my $status = $termGrp->status){
@@ -291,13 +299,15 @@ sub as_xml {
                         last_child => $term_el);
                 }
 
-            } # end termGroup
-        } # end langGroup
-    } # end entry
+            } # end tig
+        } # end langSet
+    } # end termEntry
 
     # return pretty-printed string
     XML::Twig->set_pretty_print('indented');
-    return \$root->sprint;
+    my $TBXmin = \$root->sprint;
+	$$TBXmin =~ s/>[\s\t\n]*<!--/><!--/g;  #force comments to be on same line as their parent
+	return $TBXmin;
 }
 
 ######################
@@ -356,20 +366,20 @@ sub _languages{
     return 1;
 }
 
-# add a new concept entry to the list of those found in this file
-sub _conceptStart {
+# add a new termEntry to the list of those found in this file
+sub _termEntryStart {
     my ($twig, $node) = @_;
-    my $concept = TBX::Min::Entry->new();
+    my $entry = TBX::Min::TermEntry->new();
     if($node->att('id')){
-        $concept->id($node->att('id'));
+        $entry->id($node->att('id'));
     }else{
-        carp 'found entry missing id attribute';
+        carp 'found termEntry missing id attribute';
     }
-    push @{ $twig->{tbx_min_entries} }, $concept;
+    push @{ $twig->{tbx_min_entries} }, $entry;
     return 1;
 }
 
-#just set the subject_field of the current concept
+#just set the subject_field of the current termEntry
 sub _subjectField {
     my ($twig, $node) = @_;
     $twig->{tbx_min_entries}->[-1]->
@@ -377,15 +387,15 @@ sub _subjectField {
     return 1;
 }
 
-# Create a new LangGroup, add it to the current concept,
-# and set it as the current LangGroup.
+# Create a new langSet, add it to the current termEntry,
+# and set it as the current langSet.
 sub _langStart {
     my ($twig, $node) = @_;
-    my $lang = TBX::Min::LangGroup->new();
+    my $lang = TBX::Min::LangSet->new();
     if($node->att('xml:lang')){
         $lang->code($node->att('xml:lang'));
     }else{
-        carp 'found langGroup missing xml:lang attribute';
+        carp 'found langSet missing xml:lang attribute';
     }
 
     $twig->{tbx_min_entries}->[-1]->add_lang_group($lang);
@@ -393,47 +403,67 @@ sub _langStart {
     return 1;
 }
 
-# Create a new termGroup, add it to the current langGroup,
-# and set it as the current termGroup.
+# Create a new tig, add it to the current langSet,
+# and set it as the current tig.
 sub _termGrpStart {
     my ($twig) = @_;
-    my $term = TBX::Min::TermGroup->new();
+    my $term = TBX::Min::TIG->new();
     $twig->{tbx_min_current_lang_grp}->add_term_group($term);
     $twig->{tbx_min_current_term_grp} = $term;
     return 1;
 }
 
+# Create a new noteGrp, add it to the current tig,
+# and set it as the curren noteGrp.
+sub _noteGrpStart {
+	my ($twig) = @_;
+	my $note_grp = TBX::Min::NoteGrp->new();
+	$twig->{tbx_min_current_term_grp}->add_note_group($note_grp);
+	$twig->{tbx_min_current_note_grp} = $note_grp;
+	return 1;
+}
+
+sub _noteStart {
+    my ($twig) = @_;
+    my $note = TBX::Min::Note->new();
+    $twig->{tbx_min_current_note_grp}->add_note($note);
+    $twig->{tbx_min_current_note} = $note;
+    return 1;
+}
+
 1;
 
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 TBX::Min - Read, write and edit TBX-Min files
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
     use TBX::Min;
     my $min = TBX::Min->new('/path/to/file.tbx');
     my $entries = $min->entries;
-    my $entry = TBX::Min::Entry->new({id => 'B001'});
-    $min->add_entry($entry);
+    my $termEntry = TBX::Min::TermEntry->new({id => 'B001'});
+    $min->add_entry($termEntry);
 
 =head1 DESCRIPTION
 
 This module allows you to read, write and edit the contents of TBX-Min
 data.
 
-C<use>ing this module also automatically C<use>s L<TBX::Min::Entry>,
-L<TBX::Min::LangGroup>, and L<TBX::Min::TermGroup> via
-L<Import::Into>. LangGroups contain TermGroups, Entries contain
-LangGroups, and this class contains Entries. These correspond to the
+C<use>ing this module also automatically C<use>s L<TBX::Min::TermEntry>,
+L<TBX::Min::LangSet>, and L<TBX::Min::TIG> via
+L<Import::Into>. LangSets contain TIGs(Term Information Groups), termEntries contain
+LangSets, and this class contains termEntries. These correspond to the
 three levels of information found in TML. You can build up TBX::Min
 documents this way and then print them via L</as_xml>. You can also
 read an entire TBX-Min XML document for editing via L</new_from_xml>.
@@ -475,7 +505,7 @@ a hash reference which is used to initialize the object. The allowed hash
 fields are C<id>, C<description>, C<date_created>, C<creator>, C<license>,
 C<directionality>, C<source_lang> and C<target_lang>, which correspond to
 methods of the same name, and C<entries>, which should be an array reference
-containing C<TBX::Min::Entry> objects. This method croaks if
+containing C<TBX::Min::TermEntry> objects. This method croaks if
 C<date_created> is not in ISO 8601 format.
 
 =head2 C<id>
@@ -518,14 +548,14 @@ be ISO 639 and 3166 (e.g. C<en-US>, C<de>, etc.).
 
 =head2 C<entries>
 
-Returns an array ref containing the C<TBX::Min::Entry> objects contained
+Returns an array ref containing the C<TBX::Min::TermEntry> objects contained
 in the document.The array ref is the same one used to store the objects
 internally, so additions or removals from the array will be reflected in future
 calls to this method.
 
 =head2 C<add_entry>
 
-Adds the input C<TBX::Min::Entry> object to the list of language groups
+Adds the input C<TBX::Min::TermEntry> object to the list of language groups
 contained by this object.
 
 =head2 C<as_xml>
@@ -545,11 +575,15 @@ The following related modules:
 
 =over
 
-=item L<TBX::Min::Entry>
+=item L<TBX::Min::TermEntry>
 
-=item L<TBX::Min::LangGroup>
+=item L<TBX::Min::LangSet>
 
-=item L<TBX::Min::TermGroup>
+=item L<TBX::Min::TIG>
+
+=item L<TBX::Min::NoteGrp>
+
+=item L<TBX::Min::Note>
 
 =item L<Convert::TBX::Min>
 
@@ -562,11 +596,11 @@ L<GitHub|https://github.com/byutrg/TBX-Spec>.
 
 =head1 AUTHOR
 
-Nathan Glenn <garfieldnate@gmail.com>
+James Hayes <james.s.hayes@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Alan Melby.
+This software is copyright (c) 2016 by Alan Melby.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
